/************************************************************************/
/*                                                                      */
/*                      16 channel software PWM                         */
/*                                                                      */
/*              Author: P. Dannegger                                    */
/*                                                                      */
/************************************************************************/

#include "main.h"
#include "pwm16.h"


void pwm_init( void )
{
  MOSI_DDR = 1;
  SCK_DDR = 1;
  RCK_DDR = 1;
  SCK = 0;

  TCCR0A = 1<<WGM01;			// CTC mode
  TCCR0B = 1<<CS01^1<<CS00;		// XTAL / 64
  OCR0A = 255;
  TIMSK0 = 1<<OCIE0A;
}

prog_uint8_t TAB_A_P[16] = {
		0x08, 0x10, 0x04, 0x20, 0x02, 0x40, 0x80, 0x01,
		0x10, 0x20, 0x08, 0x40, 0x04, 0x80, 0x02, 0x01
		};

prog_uint8_t TAB_PWM[] = {		// logarithmic table
		1,
		1,
		2,
		3,
		4,
		6,
		7,
		9,
		12,
		15,
		19,
		24,
		31,
		39,
		50,
		63,
		78,
		100,
		127,
		160,
		202,
		255
		};			// = 1226 cycle (148Hz <at 11MHz)

#define	PWM_STEPS	(sizeof(TAB_PWM))


u8 pwm[2 * PWM_STEPS] = { 0 };		// 2 byte (16 LED) per PWM-step
u8 step;
u8 pwm_steps = PWM_STEPS;		// maximum value


void pwm_set( u8 channel, s8 val )
{
  u8 i;
  u8 mask;
  u8 *p;

  if( channel >= sizeof(TAB_A_P) )
    return;

  mask = pgm_read_byte(TAB_A_P+channel);
  p = pwm;
  if( channel >= 8 ){
    p += PWM_STEPS;
  }
  for( i = PWM_STEPS; i; p++, i-- ){
    if( val > 0 ){
      val--;
      *p |= mask;
    }else{
      *p &= ~mask;
    }
  }
}


#define pulse_SCK()  	USICR = 1<<USIWM0^1<<USITC; \
			USICR = 1<<USIWM0^1<<USITC^1<<USICLK

#define ROLLOVER( x, max )	x = ++x >= max ? 0 : x
                                        // count up and wrap around


ISR( TIM0_COMPA_vect )
{
  u8 *ptr;

  ROLLOVER( step, PWM_STEPS );
  ptr = TAB_PWM + step;			// point to Flash
  OCR0A = pgm_read_byte( ptr );
  ptr += pwm - TAB_PWM;			// point to SRAM

  USIDR = ptr[PWM_STEPS];	        // upper 8 LEDs
  pulse_SCK();
  pulse_SCK();
  pulse_SCK();
  pulse_SCK();
  pulse_SCK();
  pulse_SCK();
  pulse_SCK();
  pulse_SCK();

  OPTR18				// save 3 cycle
  USIDR = ptr[0];			// lower 8 LEDs
  pulse_SCK();
  pulse_SCK();
  pulse_SCK();
  pulse_SCK();
  pulse_SCK();
  pulse_SCK();
  pulse_SCK();
  pulse_SCK();

  RCK = 1;
  RCK = 0;
}					// 105 cycle
					// 105 * 22 / 64 / 1226 = 30%
