#include "main.h"
#include "si2c.h"


NIL( static void ssda_lo( void ))	// call/ret needed for I2C-Timing !
{
  SSDA_DDR = 1;
}


NIL( static void ssda_hi( void ))	// call/ret needed for I2C-Timing !
{
  SSDA_DDR = 0;
}


NIL( static void sscl_lo( void ))	// call/ret needed for I2C-Timing !
{
  SSCL = 0;
}


NIL( static void sscl_hi( void ))	// call/ret needed for I2C-Timing !
{
  SSCL = 1;
}


static u16 si2c_rw( u16 val )		// input: SDA = x, SCL = 0
{
  u8 i;

  for( i = 9; i; i-- ){			// data + ACK
    if( val & 0x8000 )
      ssda_hi();
    else
      ssda_lo();
    sscl_hi();
    val <<= 1;
    if( SSDA_PIN )
      val |= 0x80;
    sscl_lo();
  }
  return val;
}

// ----------------------------	I2C functions ----------------------------

u8 si2c_r( u8 ack )			// input ACK
{
  if( ack )
    ack = 0x80;				// only set bit 7 !
  return si2c_rw( 0xFF00 | ack ) >> 8;	// return data
}


u8 si2c_w( u8 val )			// input data
{
  return si2c_rw( 0x00FF | (val<<8));	// return ACK
}


u8 si2c_start( void )			// input: SDA = 1, SCL = x
{
  u8 i;

  SSCL_DDR = 1;
  SSDA_DDR = 0;
  SSDA = 0;
  for( i = 9; i; i-- ){			// try max 9 times
    sscl_hi();
    if( SSDA_PIN == 1 )			// SDA not hold low by EEPROM
      break;
    sscl_lo();				// pulse SCL
  }
  ssda_lo();				// send start
  sscl_lo();
  return i;				// 0 = sent start failed
}


u8 si2c_stop( void )			// input: SDA = 1, SCL = 0
{
  u8 i;

  for( i = 9; i; i-- )			// try max 9 times
  {
    ssda_lo();				// try sending stop
    sscl_hi();
    ssda_hi();
    if( SSDA_PIN == 1 )			// SDA not hold low by EEPROM
      break;
    sscl_lo();                          // add clock pulse
  }
  return i;				// 0 = sent stop failed
}

