//*********************	Accessing external I2C-EEPROM *********************

#include "main.h"
#include "si2c.h"


#define PAGE_SIZE	128			// AT24C512
#define	EEPROM_ADDR	0xA0			// I2C address


static u8 eeprom_addr( u16 addr )
{
  u8 i;

  for( i = 255; i; i-- ){
    si2c_start();
    si2c_w( EEPROM_ADDR );
    si2c_w( addr >> 8 );			// address high byte
    if( !si2c_w( addr ))			// address low byte
      break;					// 0 = ACK received
    _delay_us( 40 );				// timeout: 10.2ms
  }
  return i;					// 0 = timeout
}


u8 eeprom_wr( u16 addr, u8 len, u8 *dp )	// len = 0: 256 byte
{
  do{
    if( !eeprom_addr( addr ))
      return 0;                  		// failed
    do
      si2c_w( *dp++ );
    while( --len && (++addr & (PAGE_SIZE-1)));	// end or next page
    si2c_stop();
  }while( len );				// end
  return 1;					// success
}


u8 eeprom_rd( u16 addr, u8 len, u8 *dp )	// len = 0: 256 byte
{
  if( !eeprom_addr( addr ))
    return 0;					// failed
  si2c_start();					// repeat start
  si2c_w( EEPROM_ADDR + 1 );			// read mode
  do
    *dp++ = si2c_r( !--len );			// NACK on last byte
  while( len );
  return si2c_stop();				// success
}

